package com.s477603.movies.repository;

import com.s477603.movies.document.AverageRate;
import com.s477603.movies.document.RateRecord;
import com.s477603.movies.dto.RateDTO;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.jdbc.core.BeanPropertyRowMapper;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.stereotype.Repository;

@Repository
public class RatesRepository {
    @Autowired
    JdbcTemplate jdbcTemplate;

    public RateRecord getRateByMovieIdAndUserId(Integer movieId, String userId) {
       return  jdbcTemplate.queryForObject("SELECT user_id, movie_id, title, poster_path, username, rates.id, rate, creation_date FROM rates INNER JOIN users ON rates.user_id = users.id WHERE movie_id = ? AND user_id = ? LIMIT 1",
                BeanPropertyRowMapper.newInstance(RateRecord.class), movieId, userId);
    }

    public RateRecord save(String userId, RateDTO rateDTO) {
        jdbcTemplate.update("INSERT INTO rates (user_id, movie_id, title, poster_path, rate) VALUES (?, ?, ?, ?, ?)",
                userId, rateDTO.getMovieId(), rateDTO.getTitle(), rateDTO.getPosterPath(), rateDTO.getRate());

        return getRateByMovieIdAndUserId(rateDTO.getMovieId(), userId);
    }

    public RateRecord update(String userId, RateDTO rateDTO) {
        jdbcTemplate.update("UPDATE rates SET rate = ? WHERE user_id = ? AND movie_id = ?",
                rateDTO.getRate(), userId, rateDTO.getMovieId());

        return getRateByMovieIdAndUserId(rateDTO.getMovieId(), userId);
    }

    public RateRecord[] findAllByUserId(String userId) {
        return jdbcTemplate.query("SELECT user_id, movie_id, title, poster_path, username, rates.id, rate, creation_date FROM rates INNER JOIN users ON rates.user_id = users.id  WHERE user_id =  ?",
                BeanPropertyRowMapper.newInstance(RateRecord.class), userId).toArray(new RateRecord[0]);
    }

    public RateRecord[] findAllByMovieId(Integer movieId) {
        return jdbcTemplate.query("SELECT user_id, movie_id, title, poster_path, username, rates.id, rate, creation_date FROM rates INNER JOIN users ON rates.user_id = users.id WHERE movie_id = ?",
                BeanPropertyRowMapper.newInstance(RateRecord.class), movieId).toArray(new RateRecord[0]);
    }

    public RateRecord[] getAll() {
        return jdbcTemplate.query("SELECT user_id, movie_id, title, poster_path, username, rates.id, rate, creation_date FROM rates INNER JOIN users ON rates.user_id = users.id",
                BeanPropertyRowMapper.newInstance(RateRecord.class)).toArray(new RateRecord[0]);
    }

    public AverageRate[] getMoviesWithAverageRate() {
        return jdbcTemplate.query("SELECT movie_id, title, poster_path, AVG(rate) AS `rate_avg`, COUNT(rate) AS `rate_count` FROM rates GROUP BY movie_id, title, poster_path ORDER BY `rate_avg` DESC",
                BeanPropertyRowMapper.newInstance(AverageRate.class)).toArray(new AverageRate[0]);
    }

    public AverageRate getMovieWithAverageRate(Integer movieId) {
        return jdbcTemplate.queryForObject("SELECT movie_id, title, poster_path, AVG(rate) AS `rate_avg`, COUNT(rate) AS `rate_count` FROM rates WHERE movie_id = ? GROUP BY movie_id, title, poster_path LIMIT 1",
                BeanPropertyRowMapper.newInstance(AverageRate.class), movieId);
    }

    public boolean exists(String userId, Integer movieId) {
        return jdbcTemplate.queryForObject("SELECT EXISTS(SELECT 1 FROM rates WHERE user_id = ? AND movie_id = ?)",
                Boolean.class, userId, movieId);
    }
}
