package com.s477603.movies.repository;

import com.s477603.movies.document.Movie;
import com.s477603.movies.document.WatchlistRecord;
import com.s477603.movies.dto.WatchlistRecordDTO;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.jdbc.core.BeanPropertyRowMapper;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.stereotype.Repository;

import java.util.List;

@Repository
public class WatchlistRepository {

    @Autowired
    JdbcTemplate jdbcTemplate;

    public void save(WatchlistRecordDTO watchlistRecordDTO) {
        jdbcTemplate.update("INSERT INTO watchlist (user_id, movie_id, title, poster_path) VALUES (?, ?, ?, ?)",
                watchlistRecordDTO.getUserId(), watchlistRecordDTO.getMovieId(), watchlistRecordDTO.getTitle(), watchlistRecordDTO.getPosterPath());
    }

    public void delete(WatchlistRecordDTO watchlistRecordDTO) {
        jdbcTemplate.update("DELETE FROM watchlist WHERE movie_id = ? AND user_id = ?",
                watchlistRecordDTO.getMovieId(), watchlistRecordDTO.getUserId());
    }

    public WatchlistRecord[] findAllByUserId(String userId) {
        return jdbcTemplate.query("SELECT * FROM watchlist WHERE user_id = ?",
                BeanPropertyRowMapper.newInstance(WatchlistRecord.class), userId).toArray(new WatchlistRecord[0]);
    }

    public boolean exists(WatchlistRecordDTO watchlistRecordDTO) {
        return jdbcTemplate.queryForObject("SELECT EXISTS(SELECT 1 FROM watchlist WHERE user_id = ? AND movie_id = ?)",
                Boolean.class, watchlistRecordDTO.getUserId(), watchlistRecordDTO.getMovieId());
    }

    public WatchlistRecord toggleWatched(WatchlistRecordDTO watchlistRecordDTO) {
        jdbcTemplate.update("UPDATE watchlist SET watched = NOT watched WHERE user_id = ? AND movie_id = ?",
                watchlistRecordDTO.getUserId(), watchlistRecordDTO.getMovieId());

        return jdbcTemplate.queryForObject("SELECT * FROM watchlist WHERE user_id = ? AND movie_id = ?",
                BeanPropertyRowMapper.newInstance(WatchlistRecord.class), watchlistRecordDTO.getUserId(), watchlistRecordDTO.getMovieId());
    }
}
