package com.s477603.movies.service;

import com.s477603.movies.document.Movie;
import com.s477603.movies.document.MoviesCommonResponse;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.HttpMethod;
import org.springframework.http.ResponseEntity;
import org.springframework.stereotype.Service;
import org.springframework.web.client.RestTemplate;

@Service
public final class IMDBService {
    @Value("${tmdb.api-key}")
    private String apiKey;
    private final String BASE_URL = "https://api.themoviedb.org/3";

    private String createUrl(String path) {
        String apiSuffix;
        if (path.contains("?")) {
            apiSuffix = "&api_key=" + apiKey;
        } else {
            apiSuffix = "?api_key=" + apiKey;
        }

        return BASE_URL + path + apiSuffix + "&language=pl-PL";
    }
    private final RestTemplate restTemplate;
    public IMDBService(RestTemplate restTemplate) {
        this.restTemplate = restTemplate;
    }

    public MoviesCommonResponse getTrending(int page) {
        ResponseEntity<MoviesCommonResponse> exchange = restTemplate.exchange(createUrl("/movie/popular?page=%d".formatted(page)), HttpMethod.GET, null, MoviesCommonResponse.class);
        return exchange.getBody();
    }

    public Movie getMovieById(int id) {
        ResponseEntity<Movie> exchange = restTemplate.exchange(createUrl("/movie/%d".formatted(id)), HttpMethod.GET, null, Movie.class);

        if (!exchange.getStatusCode().is2xxSuccessful()) {
            throw new RuntimeException("Movie not found");
        }
        return exchange.getBody();
    }

    public MoviesCommonResponse searchMovies(String query, int page) {
        ResponseEntity<MoviesCommonResponse> exchange = restTemplate.exchange(createUrl("/search/movie?query=%s&page=%o".formatted(query, page)), HttpMethod.GET, null, MoviesCommonResponse.class);
        return exchange.getBody();
    }
}
