package com.s477603.movies.web;

import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.fasterxml.jackson.dataformat.xml.XmlMapper;
import com.s477603.movies.document.Movie;
import com.s477603.movies.document.MoviesCommonResponse;
import com.s477603.movies.service.IMDBService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.core.io.ByteArrayResource;
import org.springframework.core.io.InputStreamResource;
import org.springframework.http.HttpHeaders;
import org.springframework.http.HttpStatus;
import org.springframework.http.MediaType;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileWriter;
import java.io.IOException;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;

@RestController
@RequestMapping("/api/movies")
public class MoviesController {

    @Autowired
    IMDBService imdbService;

    @GetMapping("/trending")
    public ResponseEntity trending(@RequestParam(name = "page", defaultValue = "1") int page) {
        MoviesCommonResponse trending = imdbService.getTrending(page);
        ObjectMapper mapper = new ObjectMapper();
        final HttpHeaders httpHeaders = new HttpHeaders();
        httpHeaders.setContentType(MediaType.APPLICATION_JSON);

        try {
            return new ResponseEntity<String>(mapper.writeValueAsString(trending.getResults()), httpHeaders, HttpStatus.OK);
        } catch (JsonProcessingException e) {
            throw new RuntimeException(e);
        }
    }

    @GetMapping("/trending/download")
    public ResponseEntity trendingGenerate() throws IOException {

        MoviesCommonResponse trending = imdbService.getTrending(1);
        XmlMapper xmlMapper = new XmlMapper();
        String xml = xmlMapper.writeValueAsString(trending);
        FileWriter fw = new FileWriter("trending.xml");

        fw.write(xml);

        File file = new File("trending.xml");

        HttpHeaders header = new HttpHeaders();
        header.add(HttpHeaders.CONTENT_DISPOSITION, "attachment; filename=trending.xml");
        header.add("Cache-Control", "no-cache, no-store, must-revalidate");
        header.add("Pragma", "no-cache");
        header.add("Expires", "0");

        Path path = Paths.get(file.getAbsolutePath());
        ByteArrayResource resource = new ByteArrayResource(Files.readAllBytes(path));

        return ResponseEntity.ok()
                .headers(header)
                .contentLength(file.length())
                .contentType(MediaType.parseMediaType("application/octet-stream"))
                .body(resource);
    }


    @GetMapping("/search")
    public ResponseEntity search(@RequestParam(name = "query") String query, @RequestParam(name = "page", defaultValue = "1") int page) {
        MoviesCommonResponse search = imdbService.searchMovies(query, page);
        ObjectMapper mapper = new ObjectMapper();
        final HttpHeaders httpHeaders = new HttpHeaders();
        httpHeaders.setContentType(MediaType.APPLICATION_JSON);

        try {
            return new ResponseEntity<String>(mapper.writeValueAsString(search.getResults()), httpHeaders, HttpStatus.OK);
        } catch (JsonProcessingException e) {
            throw new RuntimeException(e);
        }
    }

    @GetMapping("/movie/{id}")
    public ResponseEntity movie(@PathVariable int id) {
        Movie movie = imdbService.getMovieById(id);
        ObjectMapper mapper = new ObjectMapper();
        final HttpHeaders httpHeaders = new HttpHeaders();
        httpHeaders.setContentType(MediaType.APPLICATION_JSON);

        try {
            return new ResponseEntity<String>(mapper.writeValueAsString(movie), httpHeaders, HttpStatus.OK);
        } catch (JsonProcessingException e) {
            throw new RuntimeException(e);
        }
    }
}
