package com.s477603.movies.web;

import com.s477603.movies.document.User;
import com.s477603.movies.dto.RateDTO;
import com.s477603.movies.repository.RatesRepository;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.ResponseEntity;
import org.springframework.security.core.annotation.AuthenticationPrincipal;
import org.springframework.web.bind.annotation.*;

@RestController
@RequestMapping("/api/rates")
public class RatesController {
    @Autowired
    RatesRepository ratesRepository;


    @GetMapping("/all")
    public ResponseEntity getAll() {
        return ResponseEntity.ok(ratesRepository.getAll());
    }

    @PostMapping("/save")
    public ResponseEntity save(@AuthenticationPrincipal User user, @RequestBody RateDTO rateDTO) {
        if (ratesRepository.exists(user.getId(), rateDTO.getMovieId())) {
            return ResponseEntity.ok(ratesRepository.update(user.getId(), rateDTO));
        } else {
            return ResponseEntity.ok(ratesRepository.save(user.getId(), rateDTO));
        }
    }

    @GetMapping("/user/{userId}")
    public ResponseEntity getAllByUserId(@PathVariable String userId) {
        return ResponseEntity.ok(ratesRepository.findAllByUserId(userId));
    }

    @GetMapping("/movie/{movieId}")
    public ResponseEntity getAllByMovieId(@PathVariable Integer movieId) {
        return ResponseEntity.ok(ratesRepository.findAllByMovieId(movieId));
    }

    @GetMapping("/average")
    public ResponseEntity getMoviesWithAverageRate() {
        return ResponseEntity.ok(ratesRepository.getMoviesWithAverageRate());
    }

    @GetMapping("/average/{movieId}")
    public ResponseEntity getAverageRateByMovieId(@PathVariable Integer movieId) {
        return ResponseEntity.ok(ratesRepository.getMovieWithAverageRate(movieId));
    }
}
