package com.s477603.movies.web;

import com.s477603.movies.document.User;
import com.s477603.movies.dto.WatchlistRecordDTO;
import com.s477603.movies.repository.WatchlistRepository;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.ResponseEntity;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.security.core.annotation.AuthenticationPrincipal;
import org.springframework.web.bind.annotation.*;

@RestController
@RequestMapping("/api/watchlist")
public class WatchlistController {
    @Autowired
    WatchlistRepository watchlistRepository;

    @PostMapping("/add")
    @PreAuthorize("#watchlistRecordDTO.userId == #user.id")
    public ResponseEntity add(@AuthenticationPrincipal User user, @RequestBody WatchlistRecordDTO watchlistRecordDTO) {
        if (!watchlistRepository.exists(watchlistRecordDTO)) {
            watchlistRepository.save(watchlistRecordDTO);
        }

        return ResponseEntity.ok().build();
    }

    @PostMapping("/remove")
    @PreAuthorize("#watchlistRecordDTO.userId == #user.id")
    public ResponseEntity remove(@AuthenticationPrincipal User user, @RequestBody WatchlistRecordDTO watchlistRecordDTO) {
            watchlistRepository.delete(watchlistRecordDTO);

        return ResponseEntity.ok().build();
    }

    @GetMapping("/all/{userId}")
    public ResponseEntity all(@PathVariable String userId) {
        return ResponseEntity.ok(watchlistRepository.findAllByUserId(userId));
    }

    @PostMapping("/watched")
    @PreAuthorize("#watchlistRecordDTO.userId == #user.id")
    public ResponseEntity watched(@AuthenticationPrincipal User user, @RequestBody WatchlistRecordDTO watchlistRecordDTO) {
        return ResponseEntity.ok(watchlistRepository.toggleWatched(watchlistRecordDTO));
    }

    // Plan na jutro:
    // - zmiana DTO na klasy z documents
    // - controller na Rating
    //  - dodawanie oceny
    //  - zmiana oceny
    //  - lista filmów wg ocen, liczba ocen i śrenia
    // - detal usera z watchlistą i jego ocenami
    // -zacząć frontend, zacząć od oauth2, zacząć bez spektakularnego UI
}
